package adt.service;

import adt.client.MainNodeClient;
import adt.client.MainNodeCom;
import adt.dto.Node;
import adt.entity.Vlan;

import adt.repository.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;
import com.fasterxml.jackson.databind.*;
import lombok.extern.slf4j.Slf4j;
import java.util.List;
import java.util.ArrayList;
import adt.service.PortScannerService;

import adt.tcpdump.runner.TCPDumpRunner;
import adt.tcpdump.runner.TCPDumpRunnerImpl;
import adt.tcpdump.TCPDumpThread;
import java.lang.Thread;


@Service
@Slf4j
public class OnStartupService implements ApplicationListener<ApplicationReadyEvent> {

    @Value("${token}")
    private String token;

    @Value("${output.path}")
    private String outputPath;

    @Value("${execution.mode}")
    private String executionMode;

    private final SendAssetService sendAssetService;
    private final ScheduleService scheduleService;
    private final OsDetectorService osDetectorService;

    private final WrittingService writtingService;

    private final PortScannerService portScannerService;

    private final VlanRepository vlanRepository;
    private final HostRepository hostRepository;
    private final PortRepository portRepository;
    private final ScanInfoRepository scanInfoRepository;

    ////// MAIN NODE COMMUNICATION
    private final MainNodeClient mainNodeClient;
    private final MainNodeCom mainNodeCom;


    @Autowired
    public OnStartupService(ScanInfoRepository scanInfoRepository, PortRepository portRepository,
                            HostRepository hostRepository, PortScannerService portScannerService,
                            SendAssetService sendAssetService, ScheduleService scheduleService,
                            OsDetectorService osDetectorService, WrittingService writtingService,
                            VlanRepository vlanRepository, MainNodeClient mainNodeClient,
                            MainNodeCom mainNodeCom) {

        this.scanInfoRepository = scanInfoRepository;
        this.portRepository = portRepository;
        this.sendAssetService = sendAssetService;
        this.hostRepository = hostRepository;
        this.scheduleService = scheduleService;
        this.osDetectorService = osDetectorService;
        this.writtingService = writtingService;
        this.portScannerService = portScannerService;
        this.vlanRepository = vlanRepository;

        ////// MAIN NODE COMMUNICATION
        this.mainNodeClient = mainNodeClient;
        this.mainNodeCom = mainNodeCom;
    }


    @Override
    public void onApplicationEvent(@NonNull ApplicationReadyEvent event) {

        Node node = initialize();

        List<String[]> vlanIps = new ArrayList<String[]>();

        for(Vlan vlan: node.getVlan()){

            String [] toAdd = {vlan.getNetAddress(), vlan.getVlanName()};
            vlanIps.add(toAdd);
        }

        launchDHCPDiscovery(vlanIps, node.getIp());

        vlanRepository.saveAll(node.getVlan());

        sendAssetService.sendAssets(true, null);//, "scan");

        scheduleService.scheduleSetUp(node);
    }


    private Node initialize() {

        //Node node = mainNodeClient.initialize(token);

        Node node = mainNodeCom.initialize(token);

        if(node == null){
            log.error("[--INITIALIZATION PROCESS--] Unrecognized token. Aborting execution.");
            System.exit(0);
        }

        log.info("[--INITIALIZATION PROCESS--] Successfully obtained a scanning configuration.");

        return node;
    }


    private void launchDHCPDiscovery(List<String[]> info, String nodeIp){

        for(int i=0; i < info.size(); i++) {

            String[] infoDetail = info.get(i);
            TCPDumpThread tcpDumpThread = new TCPDumpThread(token, mainNodeCom, sendAssetService, portScannerService, nodeIp, infoDetail[0], infoDetail[1], hostRepository, portRepository, scanInfoRepository);
            tcpDumpThread.start();
        }
    }


}
